//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     term.h
// Purpose:    Term in an atomic formula (literal)
// Last Change:10.10.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	TERM_INCLUDED
#define TERM_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef ATOM_INCLUDED
#include "atom.h"
#endif

#ifndef CNST_INCLUDED
#include "cnst.h"
#endif

#ifndef VAR_INCLUDED
#include "var.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// TERM_MAGIC: Magic number (identifies objects of this class for debugging).
//-----------------------------------------------------------------------------

#ifdef VER_DEBUG
static const long TERM_MAGIC = 0x5445524DL;	// 'TERM'
#endif

//=============================================================================
// Class for Terms in Atomic Formulas (Literals).
//=============================================================================

class term_c {

	public:

//-----------------------------------------------------------------------------
// Methods:
//-----------------------------------------------------------------------------

	// Constructor (overloaded):
	term_c(var_t var);
	term_c(cnst_t cnst);
	term_c(void); // for anonymous variable

	// Destructor:
	~term_c(void);

	// Is this term a non-anonymus variable?
	inline bool_t is_var(void) const
		{ CHECK(valid(), "term_c::is_var: Bad object");
		  return(Kind == TERM_VAR); }

	// Is this term the anonymus variable?
	inline bool_t is_anon(void) const
		{ CHECK(valid(), "term_c::is_anon: Bad object");
		  return(Kind == TERM_ANON); }

	// Is this term a constant (ground term)?
	inline bool_t is_cnst(void) const
		{ CHECK(valid(), "term_c::is_cnst: Bad object");
		  return(Kind == TERM_CNST); }

	// Return the variable of this term:
	inline var_t get_var(void) const
		{ CHECK(valid(), "term_c::get_var: Bad object");
		  CHECK(Kind == TERM_VAR,
			"term_c::get_var: term isn't variable");
		  return(Data.Var); }

	// Return the constant of this term:
	inline cnst_t get_cnst(void) const
		{ CHECK(valid(), "term_c::get_cnst: Bad object");
		  CHECK(Kind == TERM_CNST,
			"term_c::get_cnst: term isn't constant");
		  return(Data.Cnst); }

	// Print this Term:
	void	print(void) const;

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity Check:
	public:
	bool_t	valid(void) const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;	// Must be "TERM_MAGIC"
#endif

//-----------------------------------------------------------------------------
// Private Members:
//-----------------------------------------------------------------------------

	private:

	// Kind: Which kind of term is this?
	enum {TERM_VAR, TERM_ANON, TERM_CNST} Kind;

	// Data: The variable or the ground term.
	union {var_t Var; cnst_t Cnst;} Data;

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef term_c *term_t;

//=============================================================================
// End of Include File:
//=============================================================================

#endif

